@@ -41,6 +41,11 @@ Just type ``easy_install hgsvn``. If easy_install is not available on your
 computer, download and uncompress the source tarball, then type ``python
 setup.py install``.
 
+*Note:* hgsvn makes use of the ElementTree library. It is bundled by default
+with Python 2.5, and the setup script should install it automatically for you
+if you are using Python 2.4. However, if you get some error messages, you might
+have to install it manually (at least one user reported he had to).
+
 --------
 Features
 --------
@@ -133,6 +138,28 @@ changes to SVN automatically.
 History
 -------
 
+hgsvn 0.1.3
+-----------
+
+Improvements:
+
+* Performance improvement with ``svn log`` command in ``hgpullsvn`` (suggested
+  by Mads Kiilerich and Daniel Berlin).
+* Less obscure error message when ``svn info`` fails while returning a
+  successful return code.
+* Two simplistic man pages added.
+
+Bug fixes:
+
+* Windows compatibility fix by Bill Baxter.
+* ``hgimportsvn`` failed when used on a whole repository.
+* Fix crash on empty commit message (also reported by Neil Martinsen-Burrell
+  and Walter Landry).
+* Handle file and directory renames properly (reported by Bill Baxter).
+* SVN allows copying from a deleted file by having mixed revisions inside the
+  working copy at commit time, but Mercurial doesn't accept it (reported by
+  Neil Martinsen-Burrell).
+
 hgsvn 0.1.2
 -----------
 
@@ -179,7 +206,7 @@ __all__ = []
 
 __author__ = 'Antoine Pitrou'
 __license__ = 'GNU General Public License (version 2 or later)'
-__versioninfo__ = (0, 1, 2)
+__versioninfo__ = (0, 1, 3)
 
 base_version = '.'.join(map(str, __versioninfo__))
 full_version = base_version
