@@ -1,4 +1,6 @@
 
+from hgsvn.errors import ExternalCommandFailed
+
 import os
 import locale
 from datetime import datetime
@@ -15,18 +17,53 @@ hg_commit_prefix = "[svn] "
 hg_exclude_options = ["-X", ".svn", "-X", "**/.svn"]
 
 
+# Windows compatibility code by Bill Baxter
+if os.name == "nt":
+    def find_program(name):
+        """
+        Find the name of the program for Popen.
+        Windows is finnicky about having the complete file name. Popen
+        won't search the %PATH% for you automatically.
+        (Adapted from ctypes.find_library)
+        """
+        # See MSDN for the REAL search order.
+        base, ext = os.path.splitext(name)
+        if ext:
+            exts = [ext]
+        else:
+            exts = ['.bat', '.exe']
+        for directory in os.environ['PATH'].split(os.pathsep):
+            for e in exts:
+                fname = os.path.join(directory, base + e)
+                if os.path.exists(fname):
+                    return fname
+        return None
+else:
+    def find_program(name):
+        """
+        Find the name of the program for Popen.
+        On Unix, popen isn't picky about having absolute paths.
+        """
+        return name
+
+
 locale_encoding = locale.getpreferredencoding()
 
 def shell_quote(s):
-    return "'" + s.replace('\\', '\\\\').replace("'", "'\"'\"'") + "'"
+    if os.name == "nt":
+        q = '"'
+    else:
+        q = "'"
+    return q + s.replace('\\', '\\\\').replace("'", "'\"'\"'") + q
 
-def _run_raw_command(cmd, args):
+def _run_raw_command(cmd, args, fail_if_stderr=False):
     cmd_string = "%s %s" % (cmd,  " ".join(map(shell_quote, args)))
     print "*", cmd_string
     pipe = Popen([cmd] + args, executable=cmd, stdout=PIPE, stderr=PIPE)
     out, err = pipe.communicate()
-    if pipe.returncode != 0:
-        raise RuntimeError("command failed with non-zero return code (%d): %s\n%s"
+    if pipe.returncode != 0 or (fail_if_stderr and err.strip()):
+        raise ExternalCommandFailed(
+            "External program failed (return code %d): %s\n%s"
             % (pipe.returncode, cmd_string, err))
     return out
 
@@ -34,13 +71,16 @@ def _run_raw_shell_command(cmd):
     print "*", cmd
     st, out = commands.getstatusoutput(cmd)
     if st != 0:
-        raise RuntimeError("command failed with non-zero return code (%d): %s:\n%s" % (st, cmd, out))
+        raise ExternalCommandFailed(
+            "External program failed with non-zero return code (%d): %s\n%s"
+            % (st, cmd, out))
     return out
 
-def run_command(cmd, args=None, bulk_args=None, encoding=None):
+def run_command(cmd, args=None, bulk_args=None, encoding=None, fail_if_stderr=False):
     """
     Run a command without using the shell.
     """
+    args = args or []
     def _transform_arg(a):
         if isinstance(a, unicode):
             a = a.encode(encoding or locale_encoding)
@@ -48,8 +88,9 @@ def run_command(cmd, args=None, bulk_args=None, encoding=None):
             a = str(a)
         return a
 
+    cmd = find_program(cmd)
     if not bulk_args:
-        return _run_raw_command(cmd, map(_transform_arg, args))
+        return _run_raw_command(cmd, map(_transform_arg, args), fail_if_stderr)
     max_args_num = 254
     i = 0
     out = ""
@@ -58,7 +99,7 @@ def run_command(cmd, args=None, bulk_args=None, encoding=None):
         sub_args = []
         for a in bulk_args[i:stop]:
             sub_args.append(_transform_arg(a))
-        out += _run_raw_command(cmd, args + sub_args)
+        out += _run_raw_command(cmd, args + sub_args, fail_if_stderr)
         i = stop
     return out
 
@@ -92,12 +133,19 @@ def run_shell_command(cmd, args=None, bulk_args=None, encoding=None):
     return out
 
 def run_hg(args=None, bulk_args=None):
+    """
+    Run a Mercurial command.
+    """
     default_args = ["--encoding", "utf-8"]
     return run_command("hg", args=default_args + (args or []),
         bulk_args=bulk_args, encoding="utf-8")
 
-def run_svn(args=None, bulk_args=None):
-    return run_command("svn", args=args, bulk_args=bulk_args)
+def run_svn(args=None, bulk_args=None, fail_if_stderr=False):
+    """
+    Run an SVN command.
+    """
+    return run_command("svn",
+        args=args, bulk_args=bulk_args, fail_if_stderr=fail_if_stderr)
 
 def skip_dirs(paths, basedir="."):
     return [p for p in paths if not os.path.isdir(os.path.join(basedir, p))]
