@@ -1,9 +1,11 @@
 
 from hgsvn.common import run_svn
+from hgsvn.errors import EmptySVNLog
 
 import os
 import time
 import calendar
+import operator
 
 try:
     from xml.etree import cElementTree as ET
@@ -64,7 +66,7 @@ def parse_svn_log_xml(xml_string):
         # in a repository.
         d['author'] = author is not None and author.text or None
         d['date'] = svn_date_to_timestamp(entry.find('date').text)
-        d['message'] = entry.find('msg').text
+        d['message'] = entry.find('msg').text or ""
         paths = d['changed_paths'] = []
         for path in entry.findall('.//path'):
             copyfrom_rev = path.get('copyfrom-rev')
@@ -113,7 +115,8 @@ def get_svn_info(svn_url_or_wc, rev_number=None):
         args = ['-r', rev_number]
     else:
         args = []
-    xml_string = run_svn(svn_info_args + args + [svn_url_or_wc])
+    xml_string = run_svn(svn_info_args + args + [svn_url_or_wc],
+        fail_if_stderr=True)
     return parse_svn_info_xml(xml_string)
 
 def svn_checkout(svn_url, checkout_dir, rev_number=None):
@@ -148,7 +151,42 @@ def get_svn_status(svn_wc):
     xml_string = run_svn(svn_status_args + args)
     return parse_svn_status_xml(xml_string, svn_wc)
 
-def get_first_svn_log_entry(svn_url, rev_number=None):
+def get_one_svn_log_entry(svn_url, rev_start, rev_end, stop_on_copy=False):
+    """
+    Get the first SVN log entry in the requested revision range.
+    """
+    entries = run_svn_log(svn_url, rev_start, rev_end, 1, stop_on_copy)
+    if entries:
+        return entries[0]
+
+    # XXX performance optimization disabled because it disturbs --stop-on-copy
+
+    #sign = rev_start <= rev_end and 1 or -1
+    #cmp_func = rev_start <= rev_end and operator.le or operator.ge
+    #interval = 1
+    #while cmp_func(rev_start, rev_end):
+        #rev_stop = rev_start + interval * sign
+        #if cmp_func(rev_end, rev_stop):
+            #rev_stop = rev_end
+        #try:
+            #entries = run_svn_log(svn_url, rev_start, rev_stop, 1, stop_on_copy)
+        #except RuntimeError:
+            ## It's likely this happens because the specific branch doesn't exist
+            ## in the request revision range, so increase the latter big time.
+            #rev_start = rev_stop + sign
+            #interval *= 50
+            #continue
+        #if entries:
+            #return entries[0]
+        #rev_start = rev_stop + sign
+        #interval *= 8
+        #entries = run_svn_log(svn_url, rev_start, rev_stop, 1, stop_on_copy)
+
+    raise EmptySVNLog("No SVN log for %s between revisions %s and %s" %
+        (svn_url, rev_start, rev_end))
+
+
+def get_first_svn_log_entry(svn_url, rev_start, rev_end):
     """
     Get the first log entry after (or at) the given revision number in an SVN branch.
     By default the revision number is set to 0, which will give you the log
@@ -158,17 +196,15 @@ def get_first_svn_log_entry(svn_url, rev_number=None):
     a copy from another branch, inspect elements of the 'changed_paths' entry
     in the returned dictionary.
     """
-    entries = run_svn_log(svn_url, rev_number or 0, "HEAD", 1, stop_on_copy=True)
-    return entries[0]
+    return get_one_svn_log_entry(svn_url, rev_start, rev_end, stop_on_copy=True)
 
-def get_last_svn_log_entry(svn_url, rev_number=None):
+def get_last_svn_log_entry(svn_url, rev_start, rev_end):
     """
     Get the last log entry before (or at) the given revision number in an SVN branch.
     By default the revision number is set to HEAD, which will give you the log
     entry corresponding to the latest commit in branch.
     """
-    entries = run_svn_log(svn_url, rev_number or "HEAD", 0, 1, stop_on_copy=True)
-    return entries[0]
+    return get_one_svn_log_entry(svn_url, rev_end, rev_start, stop_on_copy=True)
 
 
 log_duration_threshold = 10.0
@@ -183,12 +219,18 @@ def iter_svn_log_entries(svn_url, first_rev, last_rev):
     """
     cur_rev = first_rev
     chunk_length = log_min_chunk_length
+    chunk_interval_factor = 1.0
     while last_rev == "HEAD" or cur_rev <= last_rev:
         start_t = time.time()
-        entries = run_svn_log(svn_url, cur_rev, last_rev, chunk_length)
+        stop_rev = min(last_rev, cur_rev + int(chunk_length * chunk_interval_factor))
+        entries = run_svn_log(svn_url, cur_rev, stop_rev, chunk_length)
         duration = time.time() - start_t
         if not entries:
-            break
+            if stop_rev == last_rev:
+                break
+            cur_rev = stop_rev + 1
+            chunk_interval_factor *= 2.0
+            continue
         for e in entries:
             yield e
         cur_rev = e['revision'] + 1
